﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Phoenix;
using Phoenix.WorldData;

namespace Scripts.DarkParadise
{
    [RuntimeObject]
    public class CorpsesAutoopen
    {
        public const ushort DeadBody = 0x2006;

        private static bool showNames;
        private static bool autoOpen;
        private static readonly DefaultPublicEvent propertyChanged = new DefaultPublicEvent();

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CorpsesAutoopen"/> class.
        /// </summary>
        public CorpsesAutoopen()
        {
            // Pozn: Snazim se mit staticky jen uplne minimum, protoze co je v instanci sezere pri rekompilaci GC

            World.ItemUpdated += new ObjectChangedEventHandler(World_ItemUpdated);

            showNames = Config.Profile.UserSettings.GetAttribute(true, "ShowNames", "CorpsesAutoopen");
            autoOpen = Config.Profile.UserSettings.GetAttribute(false, "Enabled", "CorpsesAutoopen");

            Config.Profile.UserSettings.Loaded += new EventHandler(UserSettings_Loaded);
            Config.Profile.UserSettings.Saving += new EventHandler(UserSettings_Saving);
        }

        #region Configuration

        void UserSettings_Loaded(object sender, EventArgs e)
        {
            showNames = Config.Profile.UserSettings.GetAttribute(true, "ShowNames", "CorpsesAutoopen");
            autoOpen = Config.Profile.UserSettings.GetAttribute(true, "Enabled", "CorpsesAutoopen");

            OnPropertyChanged(EventArgs.Empty);
        }

        void UserSettings_Saving(object sender, EventArgs e)
        {
            Config.Profile.UserSettings.SetAttribute(showNames, "ShowNames", "CorpsesAutoopen");
            Config.Profile.UserSettings.SetAttribute(autoOpen, "Enabled", "CorpsesAutoopen");
        }

        public static bool ShowNames
        {
            get { return showNames; }
            set
            {
                if (value != showNames) {
                    showNames = value;
                    OnPropertyChanged(EventArgs.Empty);
                }
            }
        }

        public static bool AutoOpen
        {
            get { return autoOpen; }
            set
            {
                if (value != autoOpen) {
                    autoOpen = value;
                    OnPropertyChanged(EventArgs.Empty);
                }
            }
        }

        public static event EventHandler PropertyChanged
        {
            add { propertyChanged.AddHandler(value); }
            remove { propertyChanged.RemoveHandler(value); }
        }

        protected static void OnPropertyChanged(EventArgs e)
        {
            propertyChanged.InvokeAsync(null, e);
        }

        #endregion

        void World_ItemUpdated(object sender, ObjectChangedEventArgs e)
        {
            UOItem item = new UOItem(e.Serial);
            if (item.Graphic != DeadBody)
                return;

            if (showNames && item.Name == null) {
                item.Click();
            }

            if (autoOpen && item.Distance < 7) {
                item.Use();
            }
        }
    }
}
