﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Phoenix;
using Phoenix.WorldData;
using System.Runtime.CompilerServices;

namespace Scripts.Status
{
    [RuntimeObject]
    public class ClientTitleBar
    {
        class ItemInfo
        {
            public string Name;
            public int Amount;

            public override string ToString()
            {
                return Name + ": " + Amount;
            }
        }

        private int lastHits;
        private int lastMana;
        private int lastStam;
        private UOPlayer player;
        private readonly List<object> data = new List<object>();

        public ClientTitleBar()
        {
            Core.LoginComplete += new EventHandler(Core_LoginComplete);
            Core.Disconnected += new EventHandler(Core_Disconnected);

            if (Core.LoggedIn)
                Initialize();
        }

        void Core_Disconnected(object sender, EventArgs e)
        {
            player.Changed -= new ObjectChangedEventHandler(Player_Changed);
            data.Clear();
        }

        void Core_LoginComplete(object sender, EventArgs e)
        {
            Initialize();
        }

        private void Initialize()
        {
            data.Clear();

            data.Add("|");
            AddItem("MR", 0x0F86, 0);
            AddItem("BM", 0x0F7B, 0);
            data.Add("|");
            AddItem("GH", 0x0F0C, 0);

            player = World.Player;
            player.Changed += new ObjectChangedEventHandler(Player_Changed);

            UpdateText();
        }

        public void AddItem(string name, Graphic type, UOColor color)
        {
            var sc = new SupplyCounter(UO.Backpack, type, color);
            var item = new ItemInfo { Name = name, Amount = sc.CurrentAmount };

            data.Add(item);

            sc.AmountChanged += delegate(object sender, EventArgs e)
            {
                item.Amount = sc.CurrentAmount;
                UpdateText();
            };
        }

        void Player_Changed(object sender, ObjectChangedEventArgs e)
        {
            if (player.Hits != lastHits || player.Mana != lastMana || player.Stamina != lastStam) {
                UpdateText();

                lastHits = player.Hits;
                lastMana = player.Mana;
                lastStam = player.Stamina;
            }
        }

        private void UpdateText()
        {
            StringBuilder sb = new StringBuilder();

            // Player stats
            sb.AppendFormat("UO - {0}/{1}/{2}", player.Hits, player.Mana, player.Stamina);

            // Items
            foreach (var i in data) {
                sb.Append(" ");
                sb.Append(i);
            }

            sb.AppendFormat("|{0}", World.Player.Gold);

            Client.Text = sb.ToString();
        }
    }
}
